# -*- coding: utf-8 -*-
from os import remove
import unittest
from skoolkittest import SkoolKitTestCase

TEST_BIN = 'test-sna.bin'
TEST_CTL_C = 'test-sna-c.ctl'
TEST_CTL_G = """c 16384
c 16385
z 16389
c 32768
c 32772
c 32774
c 32777
c 32786
c 32794
z 32801
c 32811
z 32813"""

USAGE = """Usage: sna2skool.py [options] file

  Convert a binary (raw memory) file or a 48K SNA or Z80 snapshot into a skool
  file.

Options:
  -c FILE  Use FILE as the control file (default is file.ctl)
  -T FILE  Use FILE as the skool file template (default is file.sft)
  -g FILE  Generate a control file in FILE
  -h       Write hexadecimal addresses in the generated control file
  -H       Write hexadecimal addresses and operands in the disassembly
  -L       Write the disassembly in lower case
  -s ADDR  Specify the address at which to start disassembling (default=16384)
  -o ADDR  Specify the origin address of file.bin (default: 65536 - length)
  -t       Show ASCII text in the comment fields
  -r       Suppress addition of comments listing entry point referrers
  -n N     Set the max number of bytes per DEFB statement to N (default=8)
  -m M     Group DEFB blocks by addresses that divide M
  -z       Write bytes with leading zeroes in DEFB statements
  -l L     Set the max number of characters per DEFM statement to L (default=66)"""

class OptionsTest(SkoolKitTestCase):
    def test_no_arguments(self):
        cmdline, output, error = self.run_sna2skool(err_lines=True)
        self.assert_output_equal(error, USAGE.split('\n'), False)

    def test_no_options(self):
        lines = self._write_skool(TEST_BIN, 10)
        self.assertEqual(lines[0], '; Routine at 16384')
        self.assertEqual(lines[1][:10], 'c16384 RET')

    def test_t(self):
        skool = self._write_skool('-t %s' % TEST_BIN, 10)
        self.assertTrue(' 16385 LD SP,12854   ; [162]' in skool)

    def test_s(self):
        start = '24832'
        lines = self._write_skool('-s %s %s' % (start, TEST_BIN), 10)
        self.assertTrue(self._get_first_address(lines) == start)

    def test_g(self):
        ctlfile = 'test_g.ctl'
        self._write_skool('-g %s %s' % (ctlfile, TEST_BIN))
        f = open(ctlfile, 'r')
        lines = []
        for line in f:
            lines.append(line.rstrip())
        f.close()
        self.assert_output_equal(TEST_CTL_G.split('\n'), lines, False)
        remove(ctlfile)

    def test_c(self):
        lines = self._write_skool('-c %s %s' % (TEST_CTL_C, TEST_BIN), 10)
        self.assertTrue(self._get_first_address(lines) == '32768')

    def _write_skool(self, args, split=0):
        cmdline, output, error = self.run_sna2skool(args, out_lines=False)
        self.assert_no_errors(error, cmdline)
        if split:
            return [line.rstrip() for line in output.split('\n', split)]
        return output

    def _get_first_address(self, lines):
        for line in lines:
            if line.startswith('c'):
                return line[1:6]

if __name__ == '__main__':
    unittest.main()
